/*
  Bear Engine

  Copyright (C) 2005-2008 Julien Jorge, Sebastien Angibaud

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  contact: plee-the-bear@gamned.org

  Please add the tag [Bear] in the subject of your mails.
*/
/**
 * \file layer_border.tpp
 * \brief Implementation of the template methods of the
 *        bear::layer_border class.
 * \author Julien Jorge
 */
#include "engine/layer/layer.hpp"

/*----------------------------------------------------------------------------*/
/**
 * \brief Create vertical margin in the level.
 * \param height The height of the blocks.
 * \param x The X-coordinate of the blocks.
 */
template<typename Event>
void bear::layer_border::create_vertical_margin
( universe::size_type height, universe::coordinate_type x )
{
  const universe::size_type max_y = get_layer().get_size().y - m_block_width;

  universe::coordinate_type y = m_block_width;
  
  for ( ; y < max_y - height; y += height )
    new_margin_block<Event>( x, y, m_block_width, height );

  new_margin_block<Event>
    ( x, y, m_block_width, max_y - y );
} // layer_border::create_vertical_margin()

/*----------------------------------------------------------------------------*/
/**
 * \brief Create horizontal margin in the level.
 * \param width The width of the blocks.
 * \param y The Y-coordinate of the blocks.
 */
template<typename Event>
void bear::layer_border::create_horizontal_margin
( universe::size_type width, universe::coordinate_type y )
{
  const universe::size_type max_x = get_layer().get_size().x - m_block_width;

  universe::coordinate_type x = m_block_width;
  
  for ( ; x < max_x - width; x += width )
    new_margin_block<Event>( x, y, width, m_block_width );

  new_margin_block<Event>( x, y, max_x - x, m_block_width );
} // layer_border::create_horizontal_margin()

/*----------------------------------------------------------------------------*/
/**
 * \brief Add a block for the margins.
 * \param x X-coordinate of the block.
 * \param y Y-coordinate of the block.
 * \param width The width of the block.
 * \param height The height of the block.
 */
template<typename Event>
void bear::layer_border::new_margin_block
( universe::coordinate_type x, universe::coordinate_type y,
  universe::size_type width, universe::size_type height )
{
  typedef engine::level_border<Event> block_type;

  block_type* block = new block_type;

  block->set_top_left( universe::position_type(x, y) );
  block->set_real_field( "width", width );
  block->set_real_field( "height", height );
  block->fix();

  CLAW_ASSERT( block->is_valid(),
               "layer_border::new_margin_block(): the block is not valid" );

  new_item( *block );
} // layer_border::new_margin_block()
